#ifndef MYX_REDIS_CONTAINERS_HPP_
#define MYX_REDIS_CONTAINERS_HPP_

#pragma once

#include <myx/base/config.hpp>

#include <QByteArrayList>
#include <QVariantList>

namespace myx {

namespace redis {

QByteArray array( const QVariantList& parts );
QByteArrayList split( const QByteArray& buffer, int* splitLength = nullptr );
// QByteArray part( const QVariant& value );
// QByteArray packHash( const QVariantHash& hash );
// QVariantHash unpackHash( const QByteArray& message );
// QByteArray packList( QByteArrayList list );
// QByteArray packList( const QVariantList& list );
QVariantList unpack_list( const QByteArray& message );
// template< typename T >
// QByteArray packPair( T first, T second );
// template< typename T1, typename T2 >
// QByteArray packPair( T1 first, T2 second );
// template< typename T >
// std::pair< T, T > unpackPair( const QByteArray& message );
// template< typename T1, typename T2 >
// std::pair< T1, T2 > unpackPair( const QByteArray& message );


/*!
 * \brief Запаковывает пару значений в сообщение
 * \param first Первое значение
 * \param second Второе значение
 * \return Текстовое сообщение
 */
template< typename T >
QByteArray pack_pair( T first, T second )
{
	return( packList( { QVariant::fromValue( first ), QVariant::fromValue( second ) } ) );
}


/*!
 * \brief Запаковывает пару значений в сообщение
 * \param first Первое значение
 * \param second Второе значение
 * \return Текстовое сообщение
 */
template< typename T1, typename T2 >
QByteArray pack_pair( T1 first, T2 second )
{
	return( packList( { QVariant::fromValue( first ), QVariant::fromValue( second ) } ) );
}


/*!
 * \brief Распаковывает сообщение в пару однотипных значений
 * \param pair Текст сообщения
 * \return Пара значений
 */
template< typename T >
std::pair< T, T > unpack_pair( const QByteArray& message )
{
	auto list = unpack_list( message );
	if ( list.size() == 2 )
	{
		return { QVariant( list.first() ).value< T >(), QVariant( list.last() ).value< T >() };
	}
	return {};
}


/*!
 * \brief Распаковывает сообщение в пару однотипных значений
 * \param pair Текст сообщения
 * \return Пара значений
 */
template< typename T1, typename T2 >
std::pair< T1, T2 > unpack_pair( const QByteArray& message )
{
	auto list = unpack_list( message );
	if ( list.size() == 2 )
	{
		return { QVariant( list.first() ).value< T1 >(), QVariant( list.last() ).value< T2 >() };
	}
	return {};
}

} // namespace redis

} // namespace myx

#ifdef MYXLIB_HEADER_ONLY
#include "containers-inl.hpp"
#endif

#endif // ifndef MYX_REDIS_CONTAINERS_HPP_
